      TITLE   'Mastercrd.ASM'
       LIST    P=16F84
 
IO_PIN EQU 007H ;SET INPUT / OUTPUT PIN
;
IO_MASK = 0FFH & ~(1 << IO_PIN)
 
 
CY      EQU     0       ; Carry/borrow bit
DC      EQU     1       ; Digit carry
Z       EQU     2       ; Zero flag
PD      EQU     3       ; Power down
T0      EQU     4       ; Time-out bit
RP0     EQU     5       ; Register page select bits
RP1     EQU     6       ; Not used in 16C84
RP2     EQU     7       ; Not used in 16C84
INDIR   EQU     00H     ; Indirect pointer
RTCC    EQU     01H     ; Real time clock / counter
PCL     EQU     02H     ; Program counter low byte
STATUS  EQU     03H     ; Status register
FSR     EQU     04H     ; Indirect pointer
PORTA   EQU     05H     ; PORT A
PORTB   EQU     06H     ; PORT B
PORTC   EQU     07H     ; PORT C
EEDATA  EQU     08H     ; Eeprom data
EEADR   EQU     09H     ; Eeprom address
PCLATH  EQU     0AH     ; Program counter high byte
INTCON  EQU     0BH     ; Interrupt control
TRISA   EQU     085H    ; Data direction port A
TRISB   EQU     086H    ; Data direction port B
EECON1  EQU     088H    ; Eeprom control
W       EQU     0       ; W reg. as destination
F       EQU     1       ; F file as destination
 
 
SCLK    EQU     5      ;SERIAL EEPROM CLOCK LINE
SDATA   EQU     4      ;SERIAL EEPROM DATA LINE
 
rb0     equ     00H     ; i/o card 0
rb1     equ     01H     ; i/o card 1
rb2     equ     02H     ; switch card 1
rb3     equ     03H     ; switch card 1
rb4     equ     04H     ; i/o card 2
rb5     equ     05H     ; usato dal soft originale (mantenuto per compatibilita')
rb6     equ     06H     ; switch card 2
rb7     equ     07H     ; i/o CAM
 
;---------------------------------------------
; Variablen Deffinition
;---------------------------------------------

	CBLOCK  0CH
	rxtxbuf         ; buffer di ricezione trasmissione seriale
	rxtxcnt         ; contatore dei bit usato nel protocollo seriale
	chidh		; chid high byte
	chidl		; chid low byte
	provid		; provider id 
	pknum		; pk to use
	decodebuf	; pointer to buffer to decode
	nanosubl	; nano sublength, used to discriminate channels
	bytecnt         ; contatore generico
	key12		; which calculated key ahs to be returned
	taboff          ; spiazzamento nella look-up table
			; quando taboff=0 la tabella non viene consultata
	bf2_class       ; contiene la classe del comando
	bf3             
	bf4             
	bf5_inst             
	bf6_ref1        
	bf7_ref2             
	bf8_lang        ; contiene il numero di byte da ricevere ancora
	chksum          ; contiene lo xor linkage della tramissione

	temp1
	temp2
	temp3

;ckey0-ckey07	camkey in use
		ckey0
		ckey1
		ckey2
		ckey3
		ckey4
		ckey5
		ckey6
		ckey7

		date_high	; Datums High Byte
		date_low	; Datums Low Byte

;calculation buffers
;pk0-pk9 for plainkey/camkey shift ops
		pk0
		pk1
		pk2
		pk3
		pk4
		pk5
		pk6
		pk7
		pk8
		pk9
		pk10

;k00-k07 encrypted key 0
		k00
		k01
		k02
		k03
		k04
		k05
		k06
		k07
		k08

;k10-k17 encrypted key 1
		k10
		k11
		k12
		k13
		k14
		k15
		k16
		k17
		k18

;16 bytes, reuse?

                ENDC


             ORG 0000H
 
              GOTO     start
 
;##############################################################
;#   Schleife 
;##############################################################

schleife      ADDLW    01H
              BTFSS    STATUS,Z
              GOTO     schleife
              RETURN
 
;##############################################################
;#   Sende Byte
;##############################################################

sendebyte     MOVWF    rxtxbuf
              XORWF    chksum,F
              MOVLW    08H
              MOVWF    rxtxcnt
              MOVLW    0E0H
              CALL     schleife
L000B         BTFSS    PORTB,rb7
              GOTO     L000B
              BSF      STATUS,RP0
              BCF      PORTB,rb7
              BCF      STATUS,RP0
              BCF      PORTB,rb7
              MOVLW    0DBH
              CALL     schleife
L0013         RRF      rxtxbuf,F
              BTFSC    STATUS,CY
              BSF      PORTB,rb7
              BTFSS    STATUS,CY
              BCF      PORTB,rb7
              MOVLW    0DCH
              CALL     schleife
              DECFSZ   rxtxcnt,F
              GOTO     L0013
              NOP
              NOP
              BSF      PORTB,rb7
              BSF      STATUS,RP0
              BSF      PORTB,rb7
              BCF      STATUS,RP0
              MOVLW    0D0H
              CALL     schleife
              RETURN
 
;##############################################################
;#   Empfange Byte
;##############################################################

empfangebyte  MOVLW    08H
              MOVWF    rxtxcnt
L0027         BTFSS    PORTB,rb7
              GOTO     L0027
L0029         BTFSC    PORTB,rb7
              GOTO     L0029
              MOVLW    0CAH
              CALL     schleife
              NOP
L002E         BCF      STATUS,CY
              BTFSC    PORTB,rb7
              BSF      STATUS,CY
              RRF      rxtxbuf,F
              MOVLW    0DCH
              CALL     schleife
              NOP
              DECFSZ   rxtxcnt,F
              GOTO     L002E
              MOVLW    0E8H
              CALL     schleife
              MOVF     rxtxbuf,W
              XORWF    chksum,F
              RETURN
 
;##############################################################
;#   Sende String
;##############################################################

sende_string  MOVLW    03FH
              MOVWF    chksum
              MOVLW    08H
              MOVWF    EEADR
              MOVLW    bf2_class
              MOVWF    FSR
              MOVLW    01H
L0043         CALL     sendebyte
              MOVF     INDIR,W
              INCF     FSR,F
              DECFSZ   EEADR,F
              GOTO     L0043
              MOVF     bf8_lang,F
              BTFSC    STATUS,Z
              GOTO     L0054
              MOVF     taboff,F
              BTFSS    STATUS,Z
              GOTO     L004F
              RETURN
L004F         MOVF     bf8_lang,W
              MOVWF    bytecnt
L0051         CALL     L03F4
              DECFSZ   bytecnt,F
              GOTO     L0051
L0054         MOVF     chksum,W
              CALL     sendebyte
              RETURN
 
;##############################################################
;#   Skip w
;##############################################################

skipw         MOVWF    PCLATH
L0058         CALL     empfangebyte
              DECFSZ   PCLATH,F
              GOTO     L0058
              RETURN
 
;##############################################################
;#   Save Challenge
;##############################################################

savechal      MOVWF    FSR		; schreibe w in den Stackzeiger
              MOVLW    08H		; lade w mit 08H
savechal2     MOVWF    EEADR		; schreibe w in EEADR rein
L005F         CALL     empfangebyte	; hole Byte vom CAM
              MOVWF    INDIR		; schreibe das Byte in den Stack
              INCF     FSR,F		; erhhe den Stackzeiger um 1
              DECFSZ   EEADR,F		; EEADR = EEADR -1 , ist EEADR=0 dann ende
              GOTO     L005F		; ist EEADR > 0 dann wiederhole ab L005F
              RETURN			; springe zurck
 
;##############################################################
;#   Empfange String und werte ihn aus
;##############################################################

empfang_string
              MOVLW    03FH
              MOVWF    chksum
              CALL     empfangebyte	; teste ob 1. Byte 01 ist
              XORLW    01H		; wenn ja, mach weiter ansonsten
              BTFSS    STATUS,Z		; frage erneut ab
              GOTO     empfang_string
              CALL     empfangebyte	; frage 2. Byte ab
              MOVWF    bf2_class	; speichere es in bf2_calass

		MOVF	bf2_class,w
              XORLW    0FEH		; prfe ob class = FE ist
              BTFSC    STATUS,Z
              GOTO     L0079
 
		MOVF	bf2_class,w
              XORLW    0FFH		; prfe ob class = FF ist
              BTFSC    STATUS,Z
              GOTO     L0079

              GOTO     empfang_string
 
L0079         CALL     empfangebyte
              MOVWF    bf5_inst		; schreibe instrucktionsbyte in bf5_inst rein
              CALL     empfangebyte
              MOVWF    bf6_ref1		; schreibe Refferenzbyte 1 in bf6_ref1 rein
              CALL     empfangebyte
              MOVWF    bf7_ref2		; schreibe Refferenzbyte 2 in bf7_ref2 rein
              CALL     empfangebyte
              MOVWF    bf8_lang		; schreibe lnge der nachf. Nanos in bf8_lang
              MOVF     bf8_lang,F
              BTFSC    STATUS,Z
              GOTO     L00F2
 
              MOVF     bf2_class,W	; Teste ob Class = 02 und Instr. Byte = 09 sind
              MOVWF    ckey0
              MOVF     bf5_inst,W
              XORWF    ckey0,W
              XORLW    0BH
              BTFSS    STATUS,Z
              GOTO     L0098
 
              MOVF     bf7_ref2,W	; Teste ob referenzbyte2 = 00 ist
              MOVWF    EEDATA		; schreibe refferenzbyte2 in EEDATA rein
              BTFSC    STATUS,Z
              GOTO     L0094
 
L008F         MOVLW    08H		; wenn refferenzbyte nicht 00 war, dann
              SUBWF    bf8_lang,F	; berspringe die ersten 8 Bytes.
              CALL     skipw
              DECFSZ   EEDATA,F
              GOTO     L008F
 
L0094         MOVLW    08H		; speichere 8 Byte CAMKEY ab 22H ab
              SUBWF    bf8_lang,F
              MOVLW    ckey0
              CALL     savechal

L0098         MOVF     bf8_lang,W
              MOVWF    bytecnt
              MOVF     bf2_class,W	; teste ob class=05 ist, wenn ja, hohle
              XORLW    05H		; alle Daten vom Class 5 Kommando
              BTFSS    STATUS,Z
              GOTO     L00E6
 
;------------------------------------------------------------------
; Class 05 Kommando - Datenbernahme
;------------------------------------------------------------------
	      MOVLW    chidh		; Stackzeiger auf chidh legen
              MOVWF    FSR
              MOVLW    06H		; lnge des Stacks ist 6
              CALL     savechal2	; 6 Bytes Empfangsdaten dort abspeichern
              SUBWF    bytecnt,F
              DECF     bytecnt,F
L00A9         CALL     empfangebyte
              ANDLW    03FH		; frage ab, ob Nano = 00 oder 40 ist (Date)
              BTFSS    STATUS,Z
              GOTO     L00B5
 
              CALL     empfangebyte
              CALL     empfangebyte
              MOVWF    date_high	; Speichere Date Highbyte in date_high ab
              CALL     empfangebyte
              MOVWF    date_low		; Speichere Date Lowbyte in date_low ab
              MOVLW    04H
              SUBWF    nanosubl,F
              GOTO     L00DE
 
L00B5         XORLW    038H		; frage nach, ob Nano = 78 ist
              BTFSS    STATUS,Z
              GOTO     L00C5
              CALL     empfangebyte
              CALL     empfangebyte
              MOVWF    pknum
              CALL     empfangebyte
              MOVWF    key12
              MOVLW    02DH
              CALL     savechal
              MOVLW    036H
              CALL     savechal
	      MOVLW    014H
              SUBWF    nanosubl,F
              GOTO     L00DE
 
L00C5         DECF     nanosubl,F
              CALL     empfangebyte
              DECF     nanosubl,F
              ANDLW    03FH
              SUBWF    nanosubl,F
              CALL     skipw


L00DE         MOVLW    05H
              SUBWF    nanosubl,W
              BTFSS    STATUS,Z
              GOTO     L00A9
L00E2         CALL     empfangebyte
              DECFSZ   bytecnt,F
              GOTO     L00E2
              GOTO     L00F2
 

L00E6         MOVF     bf2_class,W	; frage nach, ob class=FF ist
              XORLW    0FFH
              BTFSS    STATUS,Z
              GOTO     L00EF
              MOVLW    date_high
              MOVWF    FSR
              MOVF     bytecnt,W
              CALL     savechal2
              GOTO     L00F2
L00EF         CALL     empfangebyte
              DECFSZ   bytecnt,F
              GOTO     L00EF
L00F2         CALL     empfangebyte
              MOVF     chksum,F
              BTFSS    STATUS,Z
              GOTO     empfang_string
              RETURN
 
 
 
 
start         MOVLW    0FFH
              TRIS     PORTA
              TRIS     PORTB
              MOVWF    PORTA
              MOVWF    PORTB
              MOVLW    07FH
              OPTION
              BSF      PORTB,rb7
              MOVLW    014H
              MOVWF    taboff
              MOVWF    bytecnt


		MOVLW	0FFH
		MOVWF	046H




L012C         CALL     L03F4
              DECFSZ   bytecnt,F
              GOTO     L012C
 
class_abfrage CLRF     bf3
              CLRF     bf4
              CALL     empfang_string
              MOVLW    0FFH
              XORWF    bf2_class,W	; class abfrage
              BTFSC    STATUS,Z		; ist Class = FF dann springe nach class_FF
              GOTO     class_ff
 
              MOVLW    0FEH
              XORWF    bf2_class,W	; class abfrage
              BTFSC    STATUS,Z		; ist Class = FE dann springe nach class_FE
              GOTO     class_fe

              GOTO     class_abfrage	; ist class = ?? dann springe nach class_abfrage
 
;------------------------------------------------------------------
; Class 02 Kommandos - Antwort
;------------------------------------------------------------------
class_02      MOVF     bf5_inst,W
              BTFSC    STATUS,Z
              GOTO     L01BD
 
              XORLW    01H
              BTFSC    STATUS,Z
              GOTO     L01C3
 
              XORLW    03H
              BTFSC    STATUS,Z
              GOTO     L01C8
 
              XORLW    01H
              BTFSC    STATUS,Z
              GOTO     L01D0
 
              XORLW    0AH
              BTFSC    STATUS,Z
              GOTO     L01D6
 
              XORLW    03H
              BTFSC    STATUS,Z
              GOTO     L01DC
 
              XORLW    04H
              BTFSC    STATUS,Z
              GOTO     L01E3
 
              XORLW    01FH
              BTFSC    STATUS,Z
              GOTO     L01EC
 
              XORLW    01AH
              BTFSC    STATUS,Z
              GOTO     L01CC
 
              GOTO     class_abfrage
 
L01BD         MOVLW    029H
              MOVWF    taboff
              MOVLW    014H
L01C0         MOVWF    bf8_lang
L01C1         CALL     sende_string
              GOTO     class_abfrage
 
L01C3         MOVLW    03AH
              MOVWF    taboff
              MOVLW    010H
              GOTO     L01C0
 
L01C8         MOVLW    04BH
              MOVWF    taboff
              MOVLW    010H
              GOTO     L01C0
 
L01CC         MOVLW    0A9H
              MOVWF    taboff
              MOVLW    02BH
              GOTO     L01C0
 
L01D0         MOVLW    064H
              BTFSC    bf7_ref2,rb0
              MOVLW    07DH
              MOVWF    taboff
              MOVLW    018H
              GOTO     L01C0
 
L01D6         CLRF     taboff
              MOVLW    055H
              MOVWF    bf3
              CLRF     bf8_lang
              CALL     eeprom_h02
              MOVLW    0B0H
              CALL     WRITE_EE
              MOVLW    ckey0
              MOVWF    FSR
              MOVLW    08H
              MOVWF    bytecnt
L0248         MOVF     INDIR,W
              CALL     TX
              INCF     FSR,F
              DECFSZ   bytecnt,F
              GOTO     L0248
              CALL     BSTOP
              GOTO     L01C1
 
L01DC         CLRF     bf8_lang
              MOVLW    050H
              MOVF     bf6_ref1,F
              BTFSS    STATUS,Z
              MOVLW    05EH
              MOVWF    bf3
              GOTO     L01C1
 
L01E3         CLRF     taboff
              MOVLW    040H
              MOVWF    bf8_lang
              CALL     sende_string
L01E7         MOVLW    00H
              CALL     sendebyte
              DECFSZ   bf8_lang,F
              GOTO     L01E7
              GOTO     L0300
 
L01EC         MOVLW    058H
              MOVWF    bf3
              GOTO     L01E3
 
;**********************************************
;	GENERATE CONTROL BYTE
;**********************************************
CONTROL_BYTE	RLF	PCLATH,W
		ANDLW	B'00001111'     ;MASK OUT UPPER NIBBLE
		IORLW	B'10100000'     ;COMPLETE BYTE 1010AAAD
					;A=ADRESS D=DIRECTION
					;SEND CONTROL BYTE
;**********************************************
;	TRANSMIT DATA SUBROUTINE
;**********************************************

TX		MOVWF	EEDATA		;STORE BYTE TO TX
		MOVLW	B'11001111'     ; set data,clock as outputs
		TRIS	PORTB

		CALL    TX4		;TRANSMIT 8 TIMES
		CALL    TX4
		GOTO 	BITIN

TX4		CALL    TXLP
		CALL    TXLP
		CALL    TXLP
TXLP	 	RLF	EEDATA,F	; ROTATE BIT TO CARRY
                GOTO    BITOUT_R        ; send the bit to serial EE

;**********************************************
;	RECEIVE DATA SUBROUTINE
;**********************************************
RX   		MOVLW	B'11011111'    ; make SDATA an input line/SCLK = OUT
		TRIS	PORTB
		CALL    RX4
		CALL    RX4
	        BSF     STATUS,CY      ; SET ACK BIT=1
                GOTO    BITOUT         ; TO FINISH TRANSMISSION

RX4		CALL	RXLP
     		CALL 	RXLP
		CALL	RXLP
RXLP            CALL    BITIN_R         ; READ A BIT
	 	RLF	EEDATA,F	; SHIFT CARRY TO BYTE
   		RETURN

;*********************************************
;	START BIT SUBROUTINE
;*********************************************
BSTART		BSF	PORTB,SDATA	; make sure data is high
		MOVLW	B'11001111'
		TRIS	PORTB		; set data and clock lines for output
		BSF	PORTB,SCLK	; set clock high
		NOP
		BCF	PORTB,SDATA	; data line goes low during
		GOTO	BC_END		; high clock for start bit

;**********************************************
;	STOP BIT SUBROUTINE
;**********************************************

BSTOP		BCF	PORTB,SDATA	; make sure data line is low
		MOVLW	B'11001111'
		TRIS	PORTB		; set data/clock lines as outputs
		NOP
		BSF	PORTB,SCLK	; set clock high
		NOP
		BSF	PORTB,SDATA	; data goes high while clock high
					; for stop bit
B_END		NOP
BC_END		BCF	PORTB,SCLK	; set clock low again
		RETURN

;**********************************************
;	BITOUT ROUTINE , SEND BIT WHEN CARRY SET
;**********************************************

BITOUT		MOVLW	B'11001111'     ; set data,clock as outputs
		TRIS	PORTB
BITOUT_R	BTFSS	STATUS,CY	; check for state of data bit to xmit
		BCF	PORTB,SDATA	; set data line low
		BTFSC	STATUS,CY
		BSF	PORTB,SDATA	; high? set data line high

CLKOUT		BSF	PORTB,SCLK	; set clock line high
		GOTO	B_END

;**********************************************
;	BITIN ROUTINE
;**********************************************
BITIN		MOVLW	 B'11011111'    ; make SDATA an input line/SCLK = OUT
		TRIS	 PORTB
BITIN_R		BSF	 PORTB,SCLK	; set clock line high
		NOP
		BSF	 STATUS,CY	; assume input bit is high
		BTFSS	 PORTB,SDATA	; read the data bit
		BCF	 STATUS,CY	; input bit was low ,clear CARRY
		GOTO	 BC_END
 
;######################################################
;	EEPROM READ ROUTINE
;######################################################
;		Used reg: EEADR, EEDATA,W
;		IN  : W= READ ADRESS
;		OUT : W= BYTE

READ_EE
 	MOVWF	EEADR		; store byte in reg EEADR
	CALL	BSTART		; generate start bit
	BCF	STATUS,CY	; WRITE SEQUENCE
	CALL	CONTROL_BYTE	; CALC CONTROL BYTE AND TRANSMIT
	MOVF	EEADR,W 	; get word address....
	CALL	TX		; and send it
	CALL	BSTART		; generate start bit
	BSF	STATUS,CY	; READ SEQUENCE
	CALL	CONTROL_BYTE	; CALC CONTROL BYTE AND TRANSMIT
	CALL	RX		; read 1 byte from serial EE
	CALL	BSTOP		; send stop bit to end transmission
	MOVF	EEDATA,W	; GET RECEIVED BYTE
	RETURN
 
;######################################################
;	EEPROM WRITE ROUTINE
;######################################################

WRITE_EE      MOVWF    EEADR
              CALL     BSTART
              BCF      STATUS,CY
              CALL     CONTROL_BYTE
              MOVF     EEADR,W
              GOTO     TX
 
 
;######################################################
;	CC Left Routine
;######################################################
ccleft        MOVLW    07H
              MOVWF    EEDATA
              MOVLW    029H
              MOVWF    FSR
              BCF      STATUS,CY
              BTFSC    ckey0,rb7
              BSF      STATUS,CY
              RLF      INDIR,F
L0256         DECF     FSR,F
              RLF      INDIR,F
              DECFSZ   EEDATA,F
              GOTO     L0256
              RETURN
 
;######################################################
;	CC Right Routine
;######################################################
ccright_07    MOVLW    07H
              GOTO     ccright
ccright_09    MOVLW    09H
ccright       MOVWF    EEDATA
              MOVWF    FSR
              MOVLW    ckey0
              ADDWF    FSR,F
              BCF      STATUS,CY
              BTFSC    INDIR,rb0
              BSF      STATUS,CY
              MOVLW    ckey0
              MOVWF    FSR
              RRF      INDIR,F
L0268         INCF     FSR,F
              RRF      INDIR,F
              DECFSZ   EEDATA,F
              GOTO     L0268
              RETURN
 
L026D         MOVWF    bf7_ref2
              CLRF     PCLATH
              BTFSS    bf7_ref2,rb0
              BSF      PCLATH,rb0
              MOVF     012H,W
              MOVWF    FSR
              MOVF     bf3,W
              ADDWF    FSR,F
              MOVF     INDIR,W
              XORWF    bf7_ref2,W
              CALL     READ_EE
              MOVWF    bf7_ref2
              MOVF     012H,W
              MOVWF    FSR
              MOVF     bf4,W
              ADDWF    FSR,F
              MOVF     bf7_ref2,W
              XORWF    INDIR,F
	      MOVLW    02H
              ADDWF    bf5_inst,W
              ADDWF    bf3,F
              MOVLW    07H
              ANDWF    bf3,F
              BTFSS    STATUS,Z
              GOTO     L028A
              MOVF     bf5_inst,W
              SUBLW    0H
	      MOVWF    bf5_inst
L028A         MOVLW    02H
              ADDWF    bf6_ref1,W
              ADDWF    bf4,F
              MOVLW    07H
              ANDWF    bf4,F
              BTFSS    STATUS,Z
              RETURN
              MOVF     bf6_ref1,W
              SUBLW    0H
              MOVWF    bf6_ref1
              RETURN
 
;------------------------------------------------------------------
; Class 05 Kommandos - Antwort
;------------------------------------------------------------------
class_05      
		MOVLW	03H
		MOVWF	01FH
		MOVWF	PCLATH
		CLRF	020H
nextprov	MOVF	020H,W
		Call	READ_EE
       		MOVWF   rxtxbuf
		BTFSC	rxtxbuf,rb7
		GOTO	no_key

		Call	inc_eeadr
		XORWF	provid,W
		BTFSS	STATUS,Z
		GOTO	next1
		Call	inc_eeadr
		XORWF	pknum,W
		BTFSS	STATUS,Z
		GOTO	next2

		BTFSS	rxtxbuf,rb0
		GOTO	skip_chid

		Call	inc_eeadr
		XORWF	chidh,W
		BTFSS	STATUS,Z
		GOTO	next3
		Call	inc_eeadr
		XORWF	chidl,W
		BTFSS	STATUS,Z
		GOTO	next4
		INCF	020H
		GOTO	class05_key

skip_chid	INCF	020H
		INCF	020H
		INCF	020H
		GOTO	class05_key


inc_eeadr	INCF	020H
		MOVF	020H,W
		Call	READ_EE
		RETURN

next1		INCF	020H
next2		INCF	020H
next3		INCF	020H
next4		MOVLW	0CH
		ADDWF	020H,F
		BTFSS	STATUS,CY
		GOTO	nextprov
		GOTO	no_key

class05_key


;****************************************** NEU **********************************
		BTFSS	bf7_ref2,rb1
		goto	class_05_1
		MOVF	bf7_ref2,W
		MOVWF	045H

		MOVF	01FH,W
		MOVWF	PCLATH
		MOVF 020H,W
		ANDLW	0F0H
		ADDLW 0DH
		CALL	READ_EE
		BTFSC	046H,rb7
		MOVWF	046H

		MOVF	046H,W
		XORLW 0H
		BTFSC	STATUS,Z
		GOTO	class_05_1
		DECF	046H
		goto no_key
class_05_1
		MOVLW	0FFH
		MOVWF	046H


;**********************************************************************************
	      MOVF     date_high,W
              MOVWF    rxtxbuf
              MOVF     date_low,W
              MOVWF    rxtxcnt
              MOVLW    02DH
              MOVWF    012H
              CLRF     ckey0
	      CALL     L0330
              CALL     L0355

              MOVF     rxtxbuf,W
              MOVWF    date_high
              MOVF     rxtxcnt,W
              MOVWF    date_low
              MOVLW    036H
              MOVWF    012H
              CALL     L0330
              CALL     L0355
              MOVLW    05H
              MOVWF    bf2_class
              MOVLW    09DH
              MOVWF    bf3
              CLRF     bf4
              MOVLW    038H
              MOVWF    bf5_inst

;****************************************** NEU **********************************
		MOVF	045H,W
		XORLW	02H
		MOVLW	01H
		BTFSS	STATUS,Z
		goto	w_12
w_13		BTFSS	key12,rb0
		MOVLW	00H
		goto	key_rechnen
w_12		BTFSC	key12,rb0
		MOVLW	00H
key_rechnen	MOVWF	bf6_ref1

		MOVF	045H,W
		XORLW	02H
		BTFSS	STATUS,Z
		goto	one_key
            
		MOVLW    02H
            MOVWF    bf7_ref2
		MOVLW    016H
		MOVWF    bf8_lang
		CLRF	045H
		goto	class05_weiter
one_key
              MOVLW    00H
              MOVWF    bf7_ref2
              MOVLW    00EH
              MOVWF    bf8_lang
;*********************************************************************************
		

class05_weiter

              CLRF     taboff
              CALL     sende_string
              MOVF     chidh,W
              CALL     sendebyte
              MOVF     chidl,W
              CALL     sendebyte
		MOVF	provid,W
              CALL     sendebyte
              MOVF     key12,W
              CALL     sendebyte
              MOVLW    0FFH
              CALL     sendebyte
              MOVLW    0FFH
              CALL     sendebyte

		BTFSS	bf8_lang,rb4
		goto	key_out_only
		MOVLW	02DH
		CALL	key_out
		MOVLW	036H
		CALL	key_out
		GOTO	L0300

key_out_only	BTFSC	key12,rb0
		goto	key_B
key_A		MOVLW	02DH
		CALL	key_out
		GOTO	L0300
key_B		MOVLW	036H
		CALL	key_out
		GOTO	L0300

key_out		MOVWF	FSR
		MOVLW	08H
		MOVWF	bytecnt
key_1		MOVF	INDIR,W
		CALL	sendebyte
		INCF	FSR,F
		DECFSZ	bytecnt,F
		GOTO	key_1
		RETURN

L0300		CALL	L0054
		GOTO	class_abfrage
 
no_key        MOVLW    090H
              MOVWF    bf3
L030F         MOVLW    05H
              MOVWF    bf2_class
              CLRF     bf4
              CLRF     bf5_inst
              CLRF     bf6_ref1
              MOVLW    02H
              MOVWF    bf7_ref2
              CLRF     bf8_lang
              CLRF     taboff
              CALL     sende_string
              GOTO     class_abfrage
 
L0330	      MOVF     01FH,W
              MOVWF    PCLATH
              MOVF     020H,W
              MOVWF    nanosubl
              MOVLW    ckey0
              MOVWF    FSR
              MOVLW    08H
              MOVWF    bytecnt
L0328         MOVF     nanosubl,W
              CALL     READ_EE
              MOVWF    INDIR
              INCF     FSR,F
              INCF     nanosubl,F
              DECFSZ   bytecnt,F
              GOTO     L0328
              MOVF     ckey0,W
              XORWF    date_high,F
              MOVF     023H,W
              XORWF    date_low,F
              CLRF     bf3
              MOVLW    01H
              MOVWF    bf4
              CLRF     bf5_inst
              CLRF     bf6_ref1
              MOVLW    01H
              SUBWF    bf5_inst,F
              SUBWF    bf6_ref1,F
              MOVLW    0DH
              MOVWF    bf8_lang
L033B         CALL     ccright_09
              CLRF     chksum
L0340         MOVLW    ckey0
              MOVWF    FSR
              MOVF     chksum,W
              ADDWF    FSR,F
              MOVF     INDIR,W
              CALL     L026D
              INCF     chksum,F
              BTFSS    chksum,rb3
              GOTO     L0340
              DECFSZ   bf8_lang,F
              GOTO     L034D
              RETURN
 
L034D         MOVF     date_high,W
              CALL     L026D
              MOVF     date_low,W
              CALL     L026D
              GOTO     L033B
 
L0355         CALL     eeprom_h02
              MOVLW    0B0H
              MOVWF    bf7_ref2
              MOVLW    029H
              MOVWF    FSR
              MOVLW    08H
              MOVWF    bytecnt
L035C         MOVF     bf7_ref2,W
              CALL     READ_EE
              MOVWF    INDIR
              DECF     FSR,F
              INCF     bf7_ref2,F
              DECFSZ   bytecnt,F
              GOTO     L035C
              MOVLW    07H
              MOVWF    bytecnt
L0365         CALL     ccleft
              DECFSZ   bytecnt,F
              GOTO     L0365
              MOVLW    08H
              MOVWF    bytecnt
L036A         CLRF     bf8_lang
L036B         MOVLW    ckey0
              MOVWF    FSR
              MOVF     bf8_lang,W
              ADDWF    FSR,F
              MOVF     INDIR,W
              MOVWF    bf7_ref2
              MOVF     012H,W
              ADDLW    06H
              MOVWF    FSR
              MOVF     INDIR,W
              XORWF    bf7_ref2,F
              DECF     bytecnt,W
              XORWF    bf7_ref2,W
              CLRF     PCLATH
              CALL     READ_EE
              MOVWF    bf7_ref2
              MOVF     012H,W
              MOVWF    FSR
              MOVF     INDIR,W
              MOVWF    bf6_ref1
              MOVF     012H,W
              MOVWF    FSR
              MOVF     bf7_ref2,W
              MOVWF    INDIR
              MOVLW    07H
              ADDWF    FSR,F
              MOVF     INDIR,W
              MOVWF    nanosubl
              MOVLW    07H
              SUBWF    FSR,F
              MOVF     nanosubl,W
              XORWF    INDIR,F
              MOVF     012H,W
              MOVWF    FSR
              MOVLW    06H
              ADDWF    FSR,F
              MOVF     INDIR,W
              INCF     FSR,F
              MOVWF    INDIR
              DECF     FSR,F
              MOVF     bf7_ref2,W
              MOVWF    INDIR
              DECF     FSR,F
              MOVF     INDIR,W
              INCF     FSR,F
              XORWF    INDIR,F
              MOVLW    04H
              MOVWF    chksum
L039B         DECF     FSR,F
              DECF     FSR,F
              MOVF     INDIR,W
              INCF     FSR,F
              MOVWF    INDIR
              DECFSZ   chksum,F
              GOTO     L039B
              DECF     FSR,F
              MOVF     bf6_ref1,W
              MOVWF    INDIR
              INCF     bf8_lang,F
              BTFSS    bf8_lang,rb3
              GOTO     L036B
              CALL     ccright_07
              DECFSZ   bytecnt,F
              GOTO     L036A
              RETURN

;------------------------------------------------------------------
; Class FF Kommandos
;------------------------------------------------------------------
class_ff      MOVF     bf5_inst,W
              BTFSS    STATUS,Z
              GOTO     class_abfrage
L03D3         MOVF     date_high,W
              MOVWF    PCLATH
              DECF     bf8_lang,W
              MOVWF    bytecnt
              DECF     bytecnt,F
              MOVLW    02CH
              MOVWF    FSR
L03DA         MOVF     date_low,W
              CALL     WRITE_EE
              MOVF     INDIR,W
              CALL     TX
              CALL     BSTOP
              INCF     FSR,F
              INCF     date_low,F
              CALL     schleife_9x
              DECFSZ   bytecnt,F
              GOTO     L03DA
              CLRF     bf8_lang
              CLRF     taboff
              CALL     sende_string
              GOTO     class_abfrage
 
schleife_9x   CALL     schleife
              CALL     schleife
              CALL     schleife
              CALL     schleife
              CALL     schleife
              CALL     schleife
              CALL     schleife
              CALL     schleife
              CALL     schleife
              RETURN
 
L03F4         CALL     eeprom_h02
              MOVF     bytecnt,W
              SUBWF    taboff,W
              CALL     READ_EE
              CALL     sendebyte
              RETURN

;------------------------------------------------------------------
; Class FE Kommandos
;------------------------------------------------------------------

class_fe     MOVF	bf7_ref2,W
	      MOVWF    taboff
		MOVLW		01H
		CALL		sendebyte		
		MOVLW		0FFH
		CALL		sendebyte		
		MOVLW		00H
		CALL		sendebyte				
		CALL		sendebyte
		CALL		sendebyte
		MOVLW		012H
		CALL		sendebyte
		MOVF		bf6_ref1,W
		CALL		sendebyte
		MOVF		bf7_ref2,W
		CALL		sendebyte

		MOVF	bf6_ref1,W
		MOVWF	PCLATH
		MOVLW	010H
		MOVWF	bytecnt

fe_auslesen	MOVF	taboff,W
		CALL	READ_EE
		CALL	sendebyte
		INCF	taboff,F
		DECFSZ	bytecnt,F
 		GOTO	fe_auslesen
		MOVF	bytecnt,W
		CALL	sendebyte
		GOTO	class_abfrage

;##############################################################
;#   Setze EEPROM High Byte auf 02
;##############################################################

eeprom_h02    MOVLW    02H
L03FB         MOVWF    PCLATH
              RETURN

              ORG 2007H
              DATA 3FF9H

              END
